

#include <zephyr.h>
#include <stdio.h>
#include <uart.h>
#include <string.h>

#include <at_cmd.h>
#include <lte_lc.h>
#include <net/mqtt.h>
#include <net/socket.h>
#include <net/aws_fota.h>

#include <dfu/mcuboot.h>
#include <misc/reboot.h>

//#if defined(CONFIG_BSD_LIBRARY)
#include "nrf_inbuilt_key.h"
//#endif

#if !defined(CONFIG_USE_PROVISIONED_CERTIFICATES)
#include "certificates.h"
#endif

#ifndef _MQTT__H_
#define _MQTT__H_

/*
    Remove existing certificates from memory and write new ones.
*/
int provision_certificates(void);

/**@brief Resolves the configured hostname and
 * initializes the MQTT broker structure
 */
void broker_init(struct sockaddr_storage* broker_storage, const char *hostname);

/*
    Initializes the client structure (sets up tls, buffers, 
    defined username and passwords, etc) and calls broker_init.
*/
int client_init(struct mqtt_client *client, struct sockaddr_storage* broker_storage, char *hostname);

/*
    Links file descriptor struct to client within the library. Returns pointer to that struct.
*/
int fds_init(struct mqtt_client *c, struct pollfd* fds);

/**@brief Configures modem to provide LTE link.
 *
 * Blocks until link is successfully established.
 */
void modem_configure(void);

void aws_fota_cb_handler(enum aws_fota_evt_id evt);

/*
    Subscribe to sub_topic through client c by mqtt
*/
int subscribe(struct mqtt_client* client, char* sub_topic);

/*
    Publish data to pub_topic through client c by mqtt
*/
int publish_data(struct mqtt_client *c, char* pub_topic, void *data, size_t len, enum mqtt_qos qos);

/*
    Pings broker to keep connection alive. Should be called at least CONFIG_MQTT_KEEP_ALIVE seconds.
    Prints error messages.
*/
int keep_mqtt_live(struct mqtt_client *c);

/*
    Access the state of the client's connection
    0 -> unconnected. 2 -> connecting. 6-> connected. (I think)
*/
int read_client_state(struct mqtt_client* c);

/*
    Check if device requires a reboot
*/
bool do_mqtt_reboot();

int poll_routine(struct mqtt_client* c, struct pollfd* fds);

#endif
